//+------------------------------------------------------------------+
//|                                          _My Keltner Channel.mq4 |
//|                         Copyright  2008, Winning Software Corp. |
//|                                   http://www.winningsoftware.com |
//+------------------------------------------------------------------+
// modification added to send alert when candle closes outside of keltner channel.
//---------------------------------------------------------------------
#property copyright "Copyright  2008, Winning Software Corp."
#property link      "http://www.winningsoftware.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 White
#property indicator_color3 Red
//---- input parameters
extern int       TimeFrame = 0;
extern int       Length    = 10;   // default is 21;
extern int       MA_Type   = 0;
extern int       Shift     = 0;
extern int       Price     = 0;
extern double    TimesATR  = 1.0;  // default is 4.0;
extern bool      Alerts_ON = true;
extern int       StochKPeriod1=5;
extern int       StochDPeriod1=3;
extern int       StochSlowing1=2;
extern int       Upper_Stoch_Threshold = 80;
extern int       Lower_Stoch_Threshold = 20;

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
//---- indicators
   SetIndexStyle ( 0, DRAW_LINE, STYLE_DOT );
   SetIndexBuffer( 0, ExtMapBuffer1 );
   SetIndexStyle ( 1, DRAW_LINE, STYLE_DOT );
   SetIndexBuffer( 1, ExtMapBuffer2 );
   SetIndexStyle ( 2, DRAW_LINE, STYLE_DOT );
   SetIndexBuffer( 2, ExtMapBuffer3 );
   
   SetIndexLabel ( 0, Length + " KChanUp" );
   SetIndexLabel ( 1, Length + " KChanMid");
   SetIndexLabel ( 2, Length + " KChanLow");
//----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() {
//----
   
//----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
   int    counted_bars = IndicatorCounted();
   int    limit        = Bars - counted_bars;
   int    i = 0;
   double stoch_val;
//----Check for possible errors
   if ( counted_bars < 0 ) return ( -1 );
      
//----Last counted bar will be recounted
   if ( counted_bars > 0 ) counted_bars-- ;
   
   double MA = 0;    
   double ATR =0; iATR ( NULL, TimeFrame, Length, i );
   
//---Main calculation Loop
   for (i = 0; i < limit; i++ ) { 
         
         MA  = iMA ( NULL, TimeFrame, Length, Shift, MA_Type, Price, i );
         ATR = iATR( NULL, TimeFrame, Length, i );
              
         ExtMapBuffer1[i] = MA + ATR * TimesATR ;
         
         ExtMapBuffer2[i] = MA;
                                   
         ExtMapBuffer3[i] = MA - ATR * TimesATR ;
         
         // stochastic and alert code added 6/10/08 - porvosh@yahoo.com
         
         if (NewBar() && Alerts_ON) {
           // check for the following 2 events and issue alert:
           // (1) price > upper keltner channel AND stoch >= 80
           // (2) price < lower keltner band AND stoch <= 20
           //---- calculate the stoch of last bar but don't show it
           stoch_val = iStochastic(NULL,0,StochKPeriod1,StochDPeriod1,StochSlowing1,MODE_SMA,0,MODE_MAIN,i+1);
           Check_and_Issue_Alert(iClose(NULL,0,i+1), ExtMapBuffer1[i+1], ExtMapBuffer3[i+1], stoch_val, Upper_Stoch_Threshold, Lower_Stoch_Threshold);
         }

     }                         

   
   return(0);
}
//+------------------------------------------------------------------+
bool NewBar() {
  static datetime dt = 0;
  if (Time[0] != dt) {
    dt = Time[0];
    return(true);
  }
  return(false);
}
//+------------------------------------------------------------------+
bool Check_and_Issue_Alert(double last_bar_close_price, double upper_keltner_value, double lower_keltner_value, double stoch_value, int upper_threshold, int lower_threshold) {

  if ((last_bar_close_price > upper_keltner_value) && (stoch_value >= upper_threshold)) {
    Alert("close above keltner: price[" + last_bar_close_price + "] up keltner[" + upper_keltner_value + "] stoch[" + stoch_value + "]");
    return(true);
  } else if ((last_bar_close_price < lower_keltner_value) && (stoch_value <= lower_threshold)) {
    Alert("close below keltner: price[" + last_bar_close_price + "] lower keltner[" + lower_keltner_value + "] stoch[" + stoch_value + "]");
  } else {
    // for testing only
    // Alert("not a valid signal: price[" + last_bar_close_price + "] upper[" + upper_keltner_value + "] lower[" + lower_keltner_value + "] stoch[" + stoch_value + "]");
    return(false);
  }
}
//+------------------------------------------------------------------+